@@ -22,7 +22,7 @@
 *     [9] Cortes, David. "Imputing missing values with unsupervised random trees." arXiv preprint arXiv:1911.06646 (2019).
 * 
 *     BSD 2-Clause License
-*     Copyright (c) 2019, David Cortes
+*     Copyright (c) 2020, David Cortes
 *     All rights reserved.
 *     Redistribution and use in source and binary forms, with or without
 *     modification, are permitted provided that the following conditions are met:
@@ -79,6 +79,7 @@
     #include <string>
     #include <fstream>
 #endif
+typedef void (*sig_t_)(int);
 
 /* By default, will use Mersenne-Twister for RNG, but can be switched to something faster */
 #ifdef _USE_MERSENNE_TWISTER
@@ -213,7 +214,7 @@ typedef struct IsoHPlane {
     std::vector<std::vector<double>> cat_coef;
     std::vector<int>      chosen_cat;
     std::vector<double>   fill_val;
-    std::vector<double>   fill_new;
+    std::vector<double>   fill_new; /* <- when using single categ, coef will be here */
 
     double   split_point;
     size_t   hplane_left;
@@ -818,6 +819,13 @@ void get_categs(size_t ix_arr[], int x[], size_t st, size_t end, int ncat,
 long double calculate_sum_weights(std::vector<size_t> &ix_arr, size_t st, size_t end, size_t curr_depth,
                                   std::vector<double> &weights_arr, std::unordered_map<size_t, double> &weights_map);
 void set_interrup_global_variable(int s);
+class SignalSwitcher
+{
+public:
+    sig_t_ old_sig;
+    SignalSwitcher();
+    ~SignalSwitcher();
+};
 int return_EXIT_SUCCESS();
 int return_EXIT_FAILURE();
 
@@ -920,8 +928,40 @@ void serialize_imputer(Imputer &imputer, const wchar_t *output_file_path);
 void deserialize_imputer(Imputer &output_obj, const wchar_t *input_file_path);
 #endif /* _MSC_VER */
 bool has_msvc();
+#ifdef _FOR_PYTHON
+void serialize_isoforest(IsoForest &model, void *output_file_path);
+void deserialize_isoforest(IsoForest &output_obj, void *input_file_path);
+void serialize_ext_isoforest(ExtIsoForest &model, void *output_file_path);
+void deserialize_ext_isoforest(ExtIsoForest &output_obj, void *input_file_path);
+void serialize_imputer(Imputer &imputer, void *output_file_path);
+void deserialize_imputer(Imputer &output_obj, void *input_file_path);
+bool py_should_use_char();
+#endif /* _FOR_PYTHON */
 #endif /* _ENABLE_CEREAL */
 
+/* sql.cpp */
+std::vector<std::string> generate_sql(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
+                                      std::vector<std::string> &numeric_colnames, std::vector<std::string> &categ_colnames,
+                                      std::vector<std::vector<std::string>> &categ_levels,
+                                      bool output_tree_num, bool index1, bool single_tree, size_t tree_num,
+                                      int nthreads);
+std::string generate_sql_with_select_from(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
+                                          std::string &table_from, std::string &select_as,
+                                          std::vector<std::string> &numeric_colnames, std::vector<std::string> &categ_colnames,
+                                          std::vector<std::vector<std::string>> &categ_levels,
+                                          bool index1, int nthreads);
+void generate_tree_rules(std::vector<IsoTree> *trees, std::vector<IsoHPlane> *hplanes, bool output_score,
+                         size_t curr_ix, bool index1, std::string &prev_cond, std::vector<std::string> &node_rules,
+                         std::vector<std::string> &conditions_left, std::vector<std::string> &conditions_right);
+void extract_cond_isotree(IsoForest &model, IsoTree &tree,
+                          std::string &cond_left, std::string &cond_right,
+                          std::vector<std::string> &numeric_colnames, std::vector<std::string> &categ_colnames,
+                          std::vector<std::vector<std::string>> &categ_levels);
+void extract_cond_ext_isotree(ExtIsoForest &model, IsoHPlane &hplane,
+                              std::string &cond_left, std::string &cond_right,
+                              std::vector<std::string> &numeric_colnames, std::vector<std::string> &categ_colnames,
+                              std::vector<std::vector<std::string>> &categ_levels);
+
 /* dealloc.cpp */
 void dealloc_IsoForest(IsoForest &model_outputs);
 void dealloc_IsoExtForest(ExtIsoForest &model_outputs_ext);
